!*****************************************************************************
!*                  Searching for oriented crossings 
!*           between two periodic orbits in the x-y plane
!*                 for computing their linking number 
!*
!*                                            Olivier Ménard, Rouen, July 1997
!*                        Updated by Eduardo Mendes, Belo Horizonte, July 2023
!*
!*
!*  To compile 
!*           gfortran -Warray-bounds -fbounds-check Linking.f90 -o Linking.exe
!*****************************************************************************
program Linking
    implicit none
    integer, parameter :: nmin=100
    integer :: i,compt,npt1,npt2
    real :: nb
    character(len=20) :: nfic1,nfic2
    real, allocatable :: x1(:),x2(:),y1(:),y2(:),z1(:),z2(:)
    real, allocatable :: xint(:),yint(:),zint1(:),zint2(:),pnsign(:)
    integer :: iounit, ierr, count
    character(len=128) :: line
    logical :: exit_flag
    double precision :: start, finish, elapsed

!*************************************************************************
!*                  Provide the names of the data files for the two orbits
!*************************************************************************
! Input filenames and number of points
    write(*,*) 'Enter the first file name'
    read(*,*) nfic1
    write(*,*) 'Enter the second file name'
    read(*,*) nfic2
    print*

    call cpu_time(start)

    iounit = 10
    open(newunit=iounit, file=nfic1, status='old', action='read', iostat=ierr)

    if (ierr /= 0) then
        print*, 'Error opening file ', nfic1
        stop
    end if

    npt1 = 0
    do
        ! Read a line
        read(iounit, '(A)', iostat=ierr) line
        if (ierr /= 0) exit ! Exit the loop when the end of the file is reached
        npt1 = npt1 + 1
    end do
    close(iounit)

    print*, 'Number of lines in the file ', nfic1, ':', npt1

    iounit = 20
    open(newunit=iounit, file=nfic2, status='old', action='read', iostat=ierr)

    if (ierr /= 0) then
        print*, 'Error opening file ', nfic2
        stop
    end if

    npt2 = 0
    do
        ! Read a line
        read(iounit, '(A)', iostat=ierr) line
        if (ierr /= 0) exit ! Exit the loop when the end of the file is reached
        npt2 = npt2 + 1
    end do
    close(iounit)

    print*, 'Number of lines in the file ', nfic2, ':', npt2
    print*

    allocate(x1(npt1),x2(npt2),y1(npt1),y2(npt2),z1(npt1),z2(npt2))
    allocate(xint(nmin),yint(nmin),zint1(nmin),zint2(nmin),pnsign(nmin))


    ! Reading data
    open(10,file=nfic1)
    do i=1,npt1
       read(10,*) x1(i),y1(i),z1(i)
    end do 
    close(10)

    open(20,file=nfic2)
    do i=1,npt2
       read(20,*) x2(i),y2(i),z2(i)
    end do 
    close(20)

    compt=1
    nb=0

    print*, 'xmgrace ',nfic1,nfic2,'positive.dat',' negative.dat -param &
            & liuyangUPO.par'

    call extract_linking_number(nmin,compt,npt1,npt2,nb,&
                &x1,x2,y1,y2,z1,z2,xint,yint,zint1,zint2,pnsign)

    deallocate(x1,x2,y1,y2,z1,z2)
    deallocate(xint,yint,zint1,zint2,pnsign)

    call cpu_time(finish)

    elapsed = finish - start
    print *, 'Time elapsed = ', elapsed, ' seconds.'

end program Linking

subroutine extract_linking_number(nmin,compt,npt1,npt2,nb,&
           &x1,x2,y1,y2,z1,z2,xint,yint,zint1,zint2,pnsign)
    implicit none
    integer, intent(in) :: nmin,npt1,npt2
    integer, intent(inout) :: compt
    real, intent(inout) :: nb
    real :: a1,b1,a2,b2,a3,b3,a4,b4
    real :: e1,e2,e3,e4,f1,f2,f3,f4,g1,g2,h1,h2
    real :: test1,test2,test3,c1,btest,ztest
    real :: xdiff1,xdiff2
    integer :: i,j
    real, intent(in) :: x1(npt1),x2(npt2),y1(npt1),y2(npt2),z1(npt1),z2(npt2)
    real, intent(inout) :: xint(nmin),yint(nmin),zint1(nmin),zint2(nmin),pnsign(nmin)
    logical :: exit_flag
 
    open(15,file='result')
    open(17,file='positive.dat')
    open(18,file='negative.dat')
    print* 

    exit_flag = .false.
	
    do i=1,(npt1-1)
        a1=(y1(i+1)-y1(i))/(x1(i+1)-x1(i))
        b1=y1(i)-a1*x1(i)
        do j=1,(npt2-1)
            a2=(y2(j+1)-y2(j))/(x2(j+1)-x2(j))
            b2=y2(j)-a2*x2(j)
            xint(compt)=(b2-b1)/(a1-a2)
            yint(compt)=a1*xint(compt)+b1
            e1=x1(i)-xint(compt)
            e2=x1(i+1)-xint(compt)
            e3=x2(j)-xint(compt)         
            e4=x2(j+1)-xint(compt)
            f1=y1(i)-yint(compt)
            f2=y1(i+1)-yint(compt)
            f3=y2(j)-yint(compt)
            f4=y2(j+1)-yint(compt)
            g1=e1*e2
            g2=e3*e4
            h1=f1*f2
            h2=f3*f4 
            if (((g1.le.0).and.(g2.le.0)).and.((h1.le.0).and.(h2.le.0))) then
                a3=(z1(i+1)-z1(i))/(x1(i+1)-x1(i))
                b3=z1(i)-a3*x1(i)
                zint1(compt)=a3*xint(compt)+b3
                a4=(z2(j+1)-z2(j))/(x2(j+1)-x2(j))
                b4=z2(j)-a4*x2(j)
                zint2(compt)=a4*xint(compt)+b4
                btest=b1-b2
                ztest=zint1(compt)-zint2(compt)
                c1=btest*ztest
                xdiff1=e2-e1
                xdiff2=e4-e3
                test3=xdiff1*xdiff2
                if (xint(compt).lt.0) then
                    test1=1
                else 
                    test1=-1
                endif
                if (c1.gt.0) then
                    test2=-1
                else
                    test2=1
                endif
                pnsign(compt)=test1*test2
                if (test3.lt.0) then
                    pnsign(compt)=-pnsign(compt)
                endif   
                write(15,*) compt
                write(15,*) x1(i),y1(i),z1(i) 
                write(15,*) x1(i+1),y1(i+1),z1(i+1) 
                write(15,*) x2(j),y2(j),z2(j) 
                write(15,*) x2(j+1),y2(j+1),z2(j+1) 
                write(15,*) xint(compt),yint(compt)
                if (pnsign(compt).gt.0.0) then
                    write(17,*) xint(compt),yint(compt)
                else
                    write(18,*) xint(compt),yint(compt)
                endif
                write(15,*) zint1(compt),zint2(compt)
                write(15,*) pnsign(compt)
                compt=compt+1
            endif
            if (compt > nmin) then
                exit_flag = .true.
            endif
        enddo
        if (exit_flag) then
            exit
        end if
    enddo
    if (exit_flag) then
        print*, 'Number of apparent crossings exceeded - ',nmin,' - increase nmin'
    end if
    print*, compt-1,' apparent crossings'
    nb=0
    do i=1,compt-1
        nb=nb+pnsign(i)
    enddo
    write(15,*)
    write(15,*)'linking number ',nb/2
    print*, 'Linking number = ',nb/2
    close(15)
    close(17)
    close(18)

end subroutine extract_linking_number
